<?php

namespace App\Http\Controllers\Admin;
use App\Models\Brand;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use DB;
use Illuminate\Support\Facades\Auth;
use Intervention\Image\ImageManager;
use Illuminate\Support\Facades\Storage;
use File;

class BrandController extends Controller
{
    public function create()
    {
        return view('Admin.Brands.create');
    }

    public function index()
    {
        $activeStatuses = [0, 1, 2];
        $brands = Brand::whereIn('active', $activeStatuses)->orderBy('created_at', 'desc')->get();
        return view('Admin.Brands.index', compact('brands'));
    }

    public function edit(Brand $brand){
        return view('Admin.Brands.edit', compact('brand'));
    }

    public function store(Request $request){
        $validated_data = $request->validate([
            'title' => 'required|string',
            'permalink' => 'required|string|unique:brands,permalink',
            'logo_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'cover_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'active' => 'required|integer',
        ]);
        $logged_in_user = Auth::user();
        $validated_data['created_by'] = $logged_in_user->id;
        Storage::makeDirectory('public/brands');
        if ($request->hasFile('logo_image')) {
            $image = $request->file('logo_image');

            # Generate a unique image name
            $imageName = time() . '1.' . $image->getClientOriginalExtension();

            # Move the image to a temporary location
            $image->move(public_path('temp'), $imageName);

            # Optimize the image using Intervention Image
            $manager = new ImageManager();
            $optimizedImage = $manager->make(public_path('temp') . '/' . $imageName)
                ->fit(800, 500) // Adjust the dimensions as needed
                ->encode();

            # Store the optimized image in the products folder
            $optimizedImage->save(public_path('brands') . '/' . $imageName);

            # Remove the temporary image
            unlink(public_path('temp') . '/' . $imageName);

            # Set the image path in the validated data
            $validated_data['logo_image'] = 'brands/' . $imageName;
        }
        if ($request->hasFile('cover_image')) {
            $image = $request->file('cover_image');
            $imageName = time() . '.' . $image->getClientOriginalExtension();
            $image->move(public_path('temp'), $imageName);
            $manager = new ImageManager();
            $optimizedImage = $manager->make(public_path('temp') . '/' . $imageName)
                ->fit(1920, 700) // Adjust the dimensions as needed
                ->encode();
            $optimizedImage->save(public_path('brands') . '/' . $imageName);
            unlink(public_path('temp') . '/' . $imageName);
            $validated_data['cover_image'] = 'brands/' . $imageName;
        }

        $brand = Brand::create($validated_data);    
        $metaDescriptions = $request->only([
            '_seo_title',
            '_seo_tags',
            '_seo_description',
            '_alt', 
        ]);

        $brandMeta = [];
        foreach ($metaDescriptions as $option => $value) {
            if ($value !== null) {
                $brandMeta[] = [
                    'brand_id' => $brand->id,
                    'option' => $option,
                    'value' => $value,
                ];
            }
        }

        # Insert product meta descriptions in bulk
        DB::table('brands_meta')->insert($brandMeta);

        return redirect()->route('brands.create')->with('success', 'Brand created successfully.');
    }


    public function update(Request $request, $id)
    {
        $validatedData = $request->validate([
            'title' => 'required|string',
            'permalink' => 'required|string|unique:brands,permalink,' . $id,
            'logo_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'cover_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'active' => 'required|integer',
        ]);

        $brand = Brand::find($id); 

        if (!$brand) {
            return Redirect::back()->withErrors(['Brand not found.']); 
        }

        if ($request->hasFile('logo_image')) {
            $image = $request->file('logo_image');

            # Generate a unique image name
            $imageName = time() . '1.' . $image->getClientOriginalExtension();

            # Move the image to a temporary location
            $image->move(public_path('temp'), $imageName);

            # Optimize the image using Intervention Image
            $manager = new ImageManager();
            $optimizedImage = $manager->make(public_path('temp') . '/' . $imageName)
                ->fit(800, 500) // Adjust the dimensions as needed
                ->encode();

            # Store the optimized image in the products folder
            $optimizedImage->save(public_path('brands') . '/' . $imageName);

            # Remove the temporary image
            unlink(public_path('temp') . '/' . $imageName);

            # Set the image path in the validated data
            $validatedData['logo_image'] = 'brands/' . $imageName;
        }
        if ($request->hasFile('cover_image')) {
            $image = $request->file('cover_image');
            $imageName = time() . '.' . $image->getClientOriginalExtension();
            $image->move(public_path('temp'), $imageName);
            $manager = new ImageManager();
            $optimizedImage = $manager->make(public_path('temp') . '/' . $imageName)
                ->fit(1920, 700) // Adjust the dimensions as needed
                ->encode();
            $optimizedImage->save(public_path('brands') . '/' . $imageName);
            unlink(public_path('temp') . '/' . $imageName);
            $validatedData['cover_image'] = 'brands/' . $imageName;
        }
        $brand->update($validatedData);
        $metaDescriptions = $request->only([
            '_seo_title',
            '_seo_tags',
            '_seo_description',
            '_alt', 
        ]);

        $brandMeta = [];
        foreach ($metaDescriptions as $option => $value) {
            if ($value !== null) {
                $brandMeta[] = [
                    'brand_id' => $brand->id,
                    'option' => $option,
                    'value' => $value,
                ];
            }
        }
        DB::table('brands_meta')->where('brand_id', $brand->id)->delete();
        DB::table('brands_meta')->insert($brandMeta);
        $brand->save();
        return redirect()->route('brands.edit', ['brand' => $brand])->with('success', 'Brand updated successfully.');
    }

    public function destroy($id)
    {
        $brand = Brand::find($id); 
        if (!$brand) {
            return Redirect::back()->withErrors(['Brand not found.']);
        }
        $brand->delete();
        return redirect()->route('brands.index')->with('success', 'Brand deleted successfully.');
    }

    public function updateStatus(Request $request, Brand $brand)
    {
        $request->validate([
            'active' => 'required',
        ]);

        $brand->active = $request->active;
        $brand->save();

        return response()->json([
            'status' => 200,
            'message' => 'Product status updated successfully.',
        ]);
    }

    public function trash(){
        $brands = Brand::where('active',3)->get();
        return view('Admin.Brands.trash', compact('brands'));
    }
    public function check_permalink(Request $request){
        $request->validate([
            'permalink' => 'required|unique:brands,permalink',
        ]);
        return response()->json([
           'status' => true,
        ]);
    }
}
